//
// Copyright (c) 2016, MindFusion LLC - Bulgaria.
//

using System;
using System.IO;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;


namespace MindFusion.Scheduling.WinForms.Samples.CS.MilestoneItems
{
	/// <summary>
	/// Summary description for MainForm.
	/// </summary>
	public partial class MainForm : Form
	{
		/// <summary>
		/// Initializes a new instance of the MainForm class.
		/// </summary>
		public MainForm()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			_appPath = System.IO.Path.GetDirectoryName(Application.ExecutablePath);

			_lbInfo.DocumentText = Properties.Resources.Info;
			_lbInfo.DocumentCompleted += new WebBrowserDocumentCompletedEventHandler(OnDocumentCompleted);

			_calendar.InteractiveItemType = typeof(MilestoneItem);

			// Create some milestone items
			DateTime baseDate = _calendar.GetFirstDate();

			Color[] colors = new Color[]
			{
				Color.Red,
				Color.Blue,
				Color.Magenta,
				Color.DarkViolet
			};

			Random random = new Random(DateTime.Now.Millisecond);
			for (int i = 0; i < 50; i++)
			{
				MilestoneItem item = new MilestoneItem();

				item.StartTime = baseDate.Add(TimeSpan.FromDays(random.Next() % 7));
				item.EndTime = item.StartTime;
				item.HeaderText = i.ToString();
				item.Color = colors[random.Next() % colors.Length];

				_calendar.Schedule.Items.Add(item);

				_calendar.SetItemListLane(item, random.Next() % 15);
			}
		}

		private void OnDocumentCompleted(object sender, WebBrowserDocumentCompletedEventArgs e)
		{
			if (_lbInfo.Document != null)
			{
				_lbInfo.Document.Click += Document_Click;
				_lbInfo.Document.Body.Style = "font-family: Verdana, Arial, Helvetica, sans-serif; " +
					"font-size: 7.5pt; margin: 2pt; padding: 0pt; background-color: #ffffee; cursor: default;";
			}
		}

		/// <summary>
		/// Handles the Web browser document's Click event.
		/// </summary>
		private void Document_Click(object sender, HtmlElementEventArgs e)
		{
			e.ReturnValue = true;

			HtmlElement pointed = _lbInfo.Document.GetElementFromPoint(e.ClientMousePosition);
			if (pointed == null)
				return;

			if (pointed.TagName.ToLower() == "a")
			{
				string href = pointed.GetAttribute("href").ToLower();
				if (href.StartsWith("help:"))
				{
					string topic = href.Substring(5);
					string helpFile = ResolveHelpPath(_appPath);

					if (System.IO.File.Exists(helpFile))
					{
						Help.ShowHelp(this, helpFile, HelpNavigator.Topic, topic);

						// Prevent link follow
						e.ReturnValue = false;
					}
				}
			}
		}

		private static string ResolveHelpPath(string baseDir)
		{
			string[] helpFiles = new string[]
			{
				"MindFusion.Scheduling.chm",
				"MindFusion WinForms.chm"
			};

			System.IO.DirectoryInfo dirInfo = new System.IO.DirectoryInfo(baseDir);
			int i = 0;
			while (dirInfo != null)
			{
				string helpFile = System.IO.Path.Combine(dirInfo.FullName, helpFiles[i]);
				if (System.IO.File.Exists(helpFile))
					return helpFile;

				i++;
				if (i == helpFiles.Length)
				{
					i = 0;
					dirInfo = dirInfo.Parent;
				}
			}

			return string.Empty;
		}

		private void OnAbout(object sender, System.EventArgs e)
		{
			System.Reflection.Assembly assembly = System.Reflection.Assembly.GetAssembly(
				typeof(MindFusion.Scheduling.WinForms.Calendar));

			if (assembly == null)
				return;

			Type aboutFormType = assembly.GetType(
				"MindFusion.Scheduling.WinForms.AboutForm");

			if (aboutFormType == null)
				return;

			((Form)Activator.CreateInstance(aboutFormType)).ShowDialog(this);
		}

		private void OnExit(object sender, System.EventArgs e)
		{
			this.Close();
		}


		private string _appPath;
	}

	public class MilestoneItemViewer : IItemViewer
	{
		public void Draw(ItemDrawContext context)
		{
			MilestoneItem item = context.Item as MilestoneItem;

			context.Graphics.SmoothingMode = SmoothingMode.AntiAlias;
			if (context.IsMilestone)
			{
				Rectangle b = context.Bounds;
				b.Width -= 1;
				b.Height -= 1;

				Point[] pts = new Point[]
					{
						new	Point(b.Left + b.Width / 2, b.Top),
						new	Point(b.Right, b.Top + b.Height / 2),
						new	Point(b.Left + b.Width / 2, b.Bottom),
						new	Point(b.Left, b.Top + b.Height / 2)
					};

				PathGradientBrush brush = new PathGradientBrush(pts);
				brush.CenterColor = Color.White;
				brush.SurroundColors = new Color[] { item.Color };

				context.Graphics.FillPolygon(brush, pts);
				context.Graphics.DrawPolygon(new Pen(context.Style.LineColor), pts);

				context.DrawHeaderText(ref b, true, false);

				brush.Dispose();
			}
			else
			{
				context.DrawDefault();
			}
		}
	}

	[ItemViewer(typeof(MilestoneItemViewer))]
	class MilestoneItem : Appointment
	{
		public MilestoneItem()
		{
			color = Color.Black;
		}

		public override bool AllowChangeStart
		{
			get { return false; }
		}

		public override bool AllowChangeEnd
		{
			get { return false; }
		}

		public Color Color
		{
			get { return color; }
			set { color = value; }
		}


		private Color color;
	}
}
